package Pcore::DBH::Query::pgsql;

use Pcore -class;
use DBD::Pg qw[:async];

extends qw[Pcore::DBH::Query];

1;
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::DBH::Query::pgsql

=head1 SYNOPSIS

=head1 DESCRIPTION

=cut
